
/*	Copyright (C) G. Jespersen
	See Licence.txt.*/

// 
// File not compatable with VLC version	
//
  
// Doc.h : interface of the CDoc class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_DOC_H__202DD557_E9CB_4D4D_B6CE_224239F6B07C__INCLUDED_)
#define AFX_DOC_H__202DD557_E9CB_4D4D_B6CE_224239F6B07C__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "KhanDB.h"
#include "History.h"
#include "ListFiles.h"

class CDataView;

#define PAGE_CONTENTS	1
#define PAGE_DOWNLOAD	2
#define PAGE_UPDATE		3
#define PAGE_SEARCH		4	// 10/04/2017 search


// Tell linker to link with version.lib.
// For GetVersion().
#pragma comment(linker, "/defaultlib:version.lib")

// LPARAM values for CDataView::OnUpdate().
enum{ FRAMEWORK_UPDATE = 0, INVALID_RECT, DISPLAY_STRING, RAW_STRING, 
	  RAW_STRING_EX, TOGGLE_LIST_FILES, REDRAW };

// Khan Update Worker Thread function.
UINT KhanUpdateThread(LPVOID pParam);

// Video Download Worker Thread function.
UINT VideoDownloadThread(LPVOID pParam);

// Application Update Worker Thread function.
UINT AppUpdateThread(LPVOID pParam);

// Inet Stream Test Worker Thread function. 12/13/14
// new 5/20/15
//UINT TestInetStreamThread(LPVOID pParam);

// Worker Thread param.
struct THREADPARAM
{
	THREADPARAM() { Zero(); };
	void Zero()	
	{
		hwndMain	= NULL; 
		hwndDlg		= NULL;
		pEvent		= NULL;
		topicTree	= NULL;
		videoAr		= NULL;
		path		= NULL;
		fileSz		= NULL;
		numTopics	= NULL;
		version		= NULL;
		sortRoot	= FALSE;
		sortChild	= FALSE;
		sortVideo	= FALSE;
		checkUpdate = FALSE;
	};

	HWND hwndMain;
	HWND hwndDlg;
    CEvent * pEvent;
	CTopicTree * topicTree;
	CArray<CVideo, CVideo&> * videoAr;
	CString * path;
	DWORD * fileSz;
	int * numTopics;
	CString * version;
	BOOL sortRoot;
	BOOL sortChild;
	BOOL sortVideo;
	BOOL checkUpdate;
};


/////////////////////////////////////////////////////////////////////////
// Class CDoc
//
class CDoc : public CDocument
{
protected: // create from serialization only
	CDoc();
	DECLARE_DYNCREATE(CDoc)

// Attributes
public:

// Operations
public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDoc)
	public:
	virtual BOOL OnNewDocument();
	virtual void Serialize(CArchive& ar);
	virtual void OnCloseDocument();
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CDoc();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CDoc)
	afx_msg void OnSetupVideoStore();
	afx_msg void OnHelpKhan();
	afx_msg void OnApplicationUpdate();
	afx_msg void OnHistoryLastVideo();
	afx_msg void OnHistoryBackup();
	afx_msg void OnHistoryRestore();
	afx_msg void OnHistoryMerge();
	afx_msg void OnHistoryRepair();
	afx_msg void OnUpdateHistoryLastVideo(CCmdUI* pCmdUI);
	afx_msg void OnUpdateHistoryBackup(CCmdUI* pCmdUI);
	afx_msg void OnUpdateHistoryRestore(CCmdUI* pCmdUI);
	afx_msg void OnUpdateHistoryRepair(CCmdUI* pCmdUI);
	afx_msg void OnUpdateHistoryMerge(CCmdUI* pCmdUI);
	afx_msg void OnToolsOrphanVideos();
	afx_msg void OnToolsListFiles();
	afx_msg void OnUpdateToolsOrphanVideos(CCmdUI* pCmdUI);
	afx_msg void OnUpdateToolsListFiles(CCmdUI* pCmdUI);
	afx_msg void OnUpdateSetupVideoStore(CCmdUI* pCmdUI);
	//}}AFX_MSG

	// Routing to CControlsBar
	afx_msg void OnMute();
	afx_msg void OnPlayPause();
	afx_msg void OnStop();
	afx_msg void OnHideLeftPane();

	DECLARE_MESSAGE_MAP()

public:

	BOOL RouteCmdToAllViews(CView *pView, UINT nID, int nCode, void *pExtra,
							AFX_CMDHANDLERINFO *pHandlerInfo);

	// Topic data.
	CTopicTree & GetTree() { return m_tree; };
	CTopic & GetCurrentTopicRef() { return m_curTopic; };
	CVideo & GetCurrentVideoRef() { return m_curVideo; };

	// Video player.
	BOOL	GetCurrentVideoPath(CString & path, BOOL & isUrl); // new ds

	// Khan update.
	void	UpdateKhanLibrary(HWND hDlgWnd, BOOL sortRoot, BOOL sortChild, BOOL sortVideo);
	void	CancelUpdate();
	void	UpdateFinished();		// WM_USER_UPDATE_FINISHED
	void	UpdateAborted();		// WM_USER_UPDATE_ABORTED

	// Contents restore.
	BOOL	UpdateRestoredContents();

	// Video download.
	BOOL	DownloadVideos(HWND hDlgWnd);
	void	CancelVideoDownload();
	BOOL	VideoOnDisk(CVideo & video);
	BOOL	VideoForDownload(CVideo & video);
	void	RemoveVideoForDownload(CVideo & video);
	void	RemoveDownloadedVideos();
	void	AddVideoForDownload(CVideo & video);
	BOOL	GetVideoForDownload(int index, CVideo & video);
	void	RemoveAllVideosForDownload();
	int		NumVideosForDownload() { return m_vidDnLdAr.GetSize(); };
	BOOL	DownloadingVideos() { return m_hDownloadThread != NULL; };
	void	DownloadFinished();		// WM_USER_DOWNLOAD_FINISHED
	void	DownloadAborted();		// WM_USER_DOWNLOAD_ABORTED

	// Application Update
	BOOL	UpdateApplication(HWND hDlgWnd, BOOL checkForUpdate);
	void	CancelAppUpdate();
	void	AppUpdateFinished();	// WM_USER_APP_UPDATE_FINISHED
	void	AppUpdateAborted();		// WM_USER_APP_UPDATE_ABORTED
	void	CleanCanceledAppUpdate();

	// Inet Stream Test 12/13/14
	// new 5/20/15
//	BOOL	StreamCurrentVideo();
//	void	StreamTestCancel();
//	void	StreamTestFinished();				// WM_USER_STREAM_TEST_FINISHED
//	void	StreamTestAborted();				// WM_USER_STREAM_TEST_ABORTED

	// Strings for View display.
	void	SetDisplayStringsTopic(BOOL updateView = TRUE);
	void	SetDisplayStringsVideo(BOOL updateView = TRUE);

	// User data serialization
	BOOL	ReadUserData();
	BOOL	WriteUserData();

	// Utilities
	BOOL	GetVersion(CString & strVersion);
	int		CompareVersions(const CString & v1, const CString & v2);
	BOOL	IsAdmin();

private:

	// History
	void	UpdateHistoyMenu(CCmdUI* pCmdUI);

	// new ds
	CDataView * GetViewPtr();
public:
	void	SetHistoryItem(UINT seconds); 

public:
	// Video Playback ds
	BOOL m_videoPlaying;

	// Video Download Storage folder.
	CString m_videoStorePath;			// User data

	// Files List 
	CListFiles m_listFiles;

	// Fonts
	LOGFONT m_viewLf;					// User data
	LOGFONT m_treeLf;					// User data

public:
	// Khan Update worker Thread
	CEvent		m_updateEvent;
	HANDLE		m_hUpdateThread;
	CTopicTree	m_tempTopicTree;
	int			m_curNumTopics;
	THREADPARAM m_updateThreadParam;

	// Video Download Worker thread.
	CEvent		m_downloadEvent;
	HANDLE		m_hDownloadThread;
	THREADPARAM m_downloadThreadParam;

	// Application Update Thread
	CEvent		m_appUpdateEvent;
	HANDLE		m_hAppUpdateThread;
	CString		m_curVersion;
	CString		m_tempUpdatePath;
	BOOL		m_appUpdateInstalled;
	THREADPARAM m_appUpdateThreadParam;

	// Inet Stream Test Worker thread. 12/13/14
	// new 5/20/15
//	CEvent		m_streamTestEvent;
//	HANDLE		m_hStreamTestThread;
//	THREADPARAM m_streamTestThreadParam;

	// Strings for View display.
	CString m_rawStr;

	// 10/04/2017
	// Index of CTabView tab curently displayed.
	//  Initialized in CTabView::TabSelChange().
	int		  m_curTabViewIndex;

	// 10/04/2017 search
	// See: CTreeDialog::OnSetSelSearch() & CSearchDialog::OnSearchGoBtn() 
	CStringArray m_searchNodes;
	int			 m_searchVideoIndex;

	// Topic Selection data.
	CTopic	  m_curTopic;
	HTREEITEM m_hCurTopic;
	int		  m_nVideos;
	int		  m_nTotalVideoTime; // seconds
	int		  m_nChildTopics;

	// Video Selection data.
	CVideo	  m_curVideo;
	HTREEITEM m_hCurVideo;

	// History
	CHistory  m_history;
	CVideo	  m_historyLastViewed;

	// Temp file path for Khan Update
	CString m_tmpUpdateFilePath;
	DWORD	m_tmpUpdateFileSz;


private:
	// Apps Topic Tree data.
	CTopicTree m_tree;

	// Program start up flag.
	BOOL m_startUp;

	// Video download.
	CArray<CVideo, CVideo&> m_vidDnLdAr;	// User data
	CArray<CVideo, CVideo&> m_vidDnLdArTemp;
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DOC_H__202DD557_E9CB_4D4D_B6CE_224239F6B07C__INCLUDED_)
